#include "../include/aprecv-general.h"

static const char *ripcommands[]= {
	" (Unknown)",
	" (Request)",
	" (Response)",
	" (Trace On)",
	" (Trace Off)",
	" (Reserved)",
	" (Triggered Request)",
	" (Triggered Response)",
	" (Triggered Ack)",
	" (Update Request)",
	" (Update Response)",
	" (Update Ack)",
};

static const char *ripngcommands[]= {
	" (Unknown)",
	" (Request)",
	" (Response)",
};

static const char *ipxripcommands[]= {
        " (Unknown)",
        " (Request)",
        " (Response)",
};			

/* RIPv1/v2 TCP/IP Protocol Suite */
int print_rip_packet (unsigned char *packet, int opt_size) {
  static struct rip_pkt_header *rip_header;
  static struct ripv2auth_pkt_header *ripv2auth_header;
  static unsigned short address_family_ident, i, auth_type;
  static char *ip_addr, *subnet_mask, *next_hop;
 
  rip_header = (struct rip_pkt_header *)(packet+data_link_size+opt_size);
  address_family_ident = apsr_ntohs(rip_header->address_family);
  
  if(rip_header->version == 0) {
    apsr_output("- RIPv0 Header -\n  <This version is not supported/RFC 1058>");
  }
  else if (rip_header->version == 1) {
    apsr_output("- RIPv1 Header -\n  Command: %d",(int) rip_header->command);
    ip_addr = strdup(inet_ntoa(rip_header->ip_address));
    if(verbose)
	    apsr_output("%s", ripcommands[rip_header->command]);
    apsr_output("\n  Version: %d, Address family identifier: %d",(int) rip_header->version,(int) address_family_ident);
    if(verbose) {
    	    switch(address_family_ident) {
	      case APSR_RIP_IDENT_IP:
	        apsr_output(" (IP)");
	        break;
              default:
	        apsr_output(" (unknown)");
		break;
	    }
    }
    apsr_output("\n  IP Address: %s, metric: %d\n", ip_addr, apsr_ntohs(rip_header->metric));
    
    if(statistic) 
	    stats.c.ripv1++;

    free(ip_addr);
  }
  else if (rip_header->version == 2 && address_family_ident == 0xffff) {
    ripv2auth_header = (struct ripv2auth_pkt_header *)(packet+data_link_size+opt_size);
    auth_type = apsr_ntohs(ripv2auth_header->auth_type);
    apsr_output("- RIPv2 Authentication Header -\n  Command: %d",
		    (int) ripv2auth_header->command);
    if(verbose)
	    apsr_output("%s", ripcommands[ripv2auth_header->command]);
    apsr_output("\n  Version: %d, Unused: 0x%04x", 
		    (int) ripv2auth_header->version,
		    apsr_ntohs(ripv2auth_header->unused));
    apsr_output("\n  Authentication type: %u", auth_type);
    if(verbose && auth_type == 2) 
	    apsr_output(" (simple)");
    if(auth_type == 2) {
	    apsr_output("\n  Authentication: \"");
            for(i = 0; i <= 15; i++) {
		    if(isprint(ripv2auth_header->auth[i]))
			apsr_output("%c", ripv2auth_header->auth[i]);
		    else
		    	apsr_output(".");
            }			
	    apsr_output("\"\n");
    }

    if(statistic)
        stats.c.ripv2++;
  }	  
  else if (rip_header->version == 2) {	  
    apsr_output("- RIPv2 Header -\n  Command: %d",(int) rip_header->command);
    ip_addr     = strdup(inet_ntoa(rip_header->ip_address));
    subnet_mask = strdup(inet_ntoa(rip_header->subnet_mask));
    next_hop    = strdup(inet_ntoa(rip_header->next_hop_ip));
    if(verbose)
	    apsr_output("%s", ripcommands[rip_header->command]);
    apsr_output("\n  Version: %d, Routing domain: %d\n",(int) rip_header->version, apsr_ntohs(rip_header->routing_domain));
    apsr_output("  Address family identifier: %d",(int) address_family_ident);
    if(verbose) {
    	    switch(address_family_ident) {
	      case APSR_RIP_IDENT_IP:
	        apsr_output(" (IP)");
	        break;
              default:
	        apsr_output(" (unknown)");
		break;
	    }
    }
    apsr_output(", Route tag: %d\n  IP Address: %s, Subnet mask: %s\n",(int) rip_header->route_tag, ip_addr, subnet_mask);
    apsr_output("  Next hop IP: %s, Metric: %u\n", next_hop, apsr_ntohl(rip_header->metric));
  
    if(statistic) 
        stats.c.ripv2++;

    free(ip_addr);
    free(subnet_mask);
    free(next_hop);
  }
  else 
	  apsr_output(" Unknown RIP Header -\n");

  return(RET_SUCCESS);
}

/* RIPng TCP/IPv6 Protocol Suite */
int print_ripng_packet (unsigned char *packet, int opt_size) {
  static struct ripng_pkt_header *ripng_header;
 
  ripng_header = (struct ripng_pkt_header *)(packet+data_link_size+opt_size);
  
  apsr_output("- RIPng Header -\n  Command: %d", (int) ripng_header->command);
  if(verbose)
	  apsr_output("%s", ripngcommands[ripng_header->command]);
  apsr_output("\n  Version: %d\n",(int) ripng_header->version);
  
  if(statistic) 
	  stats.c.ripng++;
  
  return(RET_SUCCESS);
}

/* RIP IPX Protocol Suite */
int print_ipxrip_packet (unsigned char *packet, int opt_size) {
  static struct ipxrip_pkt_header *ipxrip_header;
  unsigned short type_operation;

  ipxrip_header = (struct ipxrip_pkt_header *)(packet+data_link_size+opt_size);
  type_operation = apsr_ntohs(ipxrip_header->type_op);

  apsr_output("\n- RIP Header -\n  Type/Operation: %d",(int) type_operation);
  if(verbose)
	  apsr_output("%s", ipxripcommands[type_operation]);

  apsr_output("\n  Network: %u", apsr_ntohl(ipxrip_header->network));
  apsr_output("\n  Hops: %d, Metric of Ticks: %d\n", apsr_ntohs(ipxrip_header->hops), apsr_ntohs(ipxrip_header->ticks));
  
  if(statistic)
	  stats.c.ipxrip++;
  
  return(RET_SUCCESS);
}  
