#include "../include/apsrlib-config.h"
#include "../include/apsrlib-vendors.h"

/* Resolve an IPv4 address and convert it to an unsigned long. */
unsigned long apsr_resolve (char *host) {
	struct hostent *hn;
	struct in_addr inaddr;
	
	if((inaddr.s_addr = inet_addr(host)) == -1) {
	  if((hn=gethostbyname(host)) == NULL) {
	    #ifdef APSRLIB_DEBUG
	    herror("Error occured while gethostbyname(host).\n");
	    #endif
	    return(RET_FAILURE);
	  }
	  memcpy((char *)&inaddr.s_addr, hn->h_addr, hn->h_length);
	}
	
	return(inaddr.s_addr);
}


/* Convert mac address (12:ff:aa:76:cc:09 for example) to an unsigned int. */
unsigned int apsr_conv_mac (char *uc_mac)
{
        typedef unsigned int *mui;
        static char mac[6];
        sscanf(uc_mac,"%2x:%2x:%2x:%2x:%2x:%2x", (mui) &mac[0], (mui) &mac[1],
                                                 (mui) &mac[2], (mui) &mac[3],
                                                 (mui) &mac[4], (mui) &mac[5]);

	return((unsigned int) &mac);
}


/* Resolve the mac address using a list of vendors. */
char *apsr_resolve_mac (unsigned char *mac)
{
        static unsigned int i, vc;
	unsigned int match=0;
	vc = ( (sizeof(vendors)) / (sizeof(struct vendorlist)) - 1);
 
        for(i = 0; i <= vc; i++) {
          if(mac[0] == vendors[i].mac[0] &&
	     mac[1] == vendors[i].mac[1] &&
	     mac[2] == vendors[i].mac[2]) {
	        match = 1;
            	break;
	  }
        }

	if(match == 0)
		return("Unknown");
	else
	        return((char *) vendors[i].name);
}

const char *ether2str(unsigned char *addr) {
	static char buf[sizeof("AP:SR:PR:OJ:EK:T0")]; /* Not thread safe */

	snprintf(buf, sizeof(buf), "%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",
	addr[0], addr[1], addr[2], addr[3], addr[4], addr[5]);

	return(buf);
}

const char *apsr_get_network_address(struct in_addr addr, int mask) {
	struct in_addr temp_addr;
	char *temp_string=NULL;
	
        temp_addr.s_addr = ntohl(addr.s_addr) >> (32 - mask);
        temp_addr.s_addr = htonl(temp_addr.s_addr << (32 - mask));
	
	temp_string = strdup(inet_ntoa(temp_addr));

	return(temp_string);
}

/* Get hardware info. */
/* FIXME: QNX/AIX/BSD support! */
#if !defined(__OSTYPE_QNX) && !defined(__OSTYPE_AIX) && !defined(__OSTYPE_FREEBSD) && !defined(__OSTYPE_NETBSD) && !defined(__OSTYPE_OPENBSD)
struct ether_addr * apsr_get_hwinfo(const char *device, int fd)
{
	static struct ifreq ifr;

	struct ether_addr *map;
	static struct ether_addr ma;

	memset(&ifr, 0, sizeof(ifr));
	map=&ma;
	strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));

	if(ioctl(fd, SIOCGIFHWADDR, (char *)&ifr) < 0) {
	  #ifdef APSRLIB_DEBUG
	    ERROR("apsr_get_hw_info: ioctl(SIOCGIFHWADDR) failed!");
	  #endif
	  return(RET_NULL);
	}
	
	memcpy(map, &ifr.ifr_hwaddr.sa_data, APSR_ETHER_ADDR_LEN);

	return(map);
}
#endif

/* Toggle promisc mode of an interface. */
int apsr_toggle_promisc (const char *device, int fd) 
{
	static struct ifreq ifr;

	strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));

	if(ioctl(fd, SIOCGIFFLAGS, &ifr) < 0) {
          #ifdef APSRLIB_DEBUG
	    ERROR("apsr_toggle_promisc - set_promisc: get ioctl(SIOCGIFFLAGS) failed!");
          #endif

	  return(RET_FAILURE);
	}

        if( (ifr.ifr_flags & IFF_PROMISC) ) {
          #ifdef APSRLIB_DEBUG
            INFO("apsr_toggle_promisc - promisc mode set - unsetting it!");
          #endif
	  
	  ifr.ifr_flags ^= IFF_PROMISC; /* Unset promisc flag. */
        } else {
	  #ifdef APSRLIB_DEBUG
	    INFO("apsr_toggle_promisc - promisc mode not set - setting it!");
	  #endif
	  
	  ifr.ifr_flags |= IFF_PROMISC;	/* Set promisc flag. */
	}

	if(ioctl(fd, SIOCSIFFLAGS, &ifr) < 0) {
	  #ifdef APSRLIB_DEBUG
	    ERROR("apsr_toggle_promisc - set_promisc: set ioctl(SIOCGIFFLAGS) failed!");
	  #endif

	  return(RET_FAILURE);
	}

	return(RET_SUCCESS);
}

int apsr_open_io_socket(void) { 
        static int fd;

        if((fd=socket(AF_INET, SOCK_DGRAM, 0)) < 0) { 
          #ifdef APSRLIB_DEBUG
            ERROR("apsr_open_io_socket: create socket failed!");
          #endif
        }

	return(fd);
}	

int apsr_close_io_socket(int fd) {
	if(close(fd) < 0) {
	  #ifdef APSRLIB_DEBUG
	    ERROR("apsr_close_io_socket: socket closing failed!");
	  #endif

	  return(RET_FAILURE);
	}

	return(RET_SUCCESS);
}

int apsr_get_link_bandwidth (char *device, int fd) {
	static struct ifreq ifr;

        #if APSRLIB_DEBUG && IFF_PORTSEL
		const char *if_port_text[][4] =
		{
		  /* Keep in step with <linux/netdevice.h> */
		  {"unknown", NULL, NULL, NULL},
		  {"10base2", "bnc", "coax", NULL},
		  {"10baseT", "utp", "tpe", NULL},
		  {"AUI", "thick", "db15", NULL},
		  {"100baseT", NULL, NULL, NULL},
		  {"100baseTX", NULL, NULL, NULL},
		  {"100baseFX", NULL, NULL, NULL},
		  {NULL, NULL, NULL, NULL},
		};
	#else
		return(RET_FAILURE);
	#endif
				  
	strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));	

	if(ioctl(fd, SIOCGIFFLAGS, &ifr) < 0) {
	  fprintf(stderr, "SIOCGIFFLAGS failed!\n");
	  return(RET_FAILURE);
	}

	#if APSRLIB_DEBUG && IFF_PORTSEL
	printf("Bandwidth: %s\n", if_port_text[ifr.ifr_ifru.ifru_map.port][0]);
	#endif

	return(RET_SUCCESS);
}


/* Wrapper functions for apsr_htons/apsr_htonl and apsr_ntohs/apsr_ntohl. */
unsigned short int apsr_htons (unsigned short int hostshort) {
	#if (__BYTE_ORDER == __BIG_ENDIAN)
	return(hostshort);
	#else
	return(htons(hostshort));
	#endif
}

unsigned long int apsr_htonl (unsigned long int hostlong) {
	#if (__BYTE_ORDER == __BIG_ENDIAN)
	return(hostlong);
	#else
	return(htonl(hostlong));
	#endif
}

unsigned short int apsr_ntohs (unsigned short int netshort) {
	#if (__BYTE_ORDER == __BIG_ENDIAN)
	return(netshort);
	#else
	return(ntohs(netshort));
	#endif
}

unsigned long int apsr_ntohl (unsigned long int netlong) {
	#if (__BYTE_ORDER == __BIG_ENDIAN)
	return(netlong);
	#else
	return(ntohl(netlong));
	#endif
}

/* Wrapper function for getservbyport */
const char *apsr_resolve_port (int port, char *proto) {
	struct servent *apsr_servent;
	char *apsr_serv;

	apsr_servent=getservbyport(htons(port), proto);

	if(apsr_servent) {
		apsr_serv=xmalloc(strlen(apsr_servent->s_name));
		strcpy(apsr_serv, apsr_servent->s_name);
	} else {
		apsr_serv=xmalloc(8);
		apsr_serv[7] = '\0';
		strncpy(apsr_serv, "unknown", 7);
	}

	return(apsr_serv);
}

/* Function to set non-blocking on a file descriptor */
void apsr_setnonblocking (int fd) {
	int fd_options;

	fd_options = fcntl(fd, F_GETFL);
	if (fd_options < 0) {
		fprintf(stderr, "Error while fetching fcntl(F_GETFL)!\n");
		exit(EXIT_FAILURE);
        }
	
	fd_options = (fd_options | O_NONBLOCK);

	if (fcntl(fd, F_SETFL, fd_options) < 0) {
		fprintf(stderr, "Error while setting fcntl(F_SETFL)!\n");
		exit(EXIT_FAILURE);
        }
}

#ifdef HAVE_ICONV_H
/* Function for UTF-8 decoding (maybe broken) */
int apsr_utf8_decode(char *return_string, const char *original_string) {
	iconv_t conversion_descriptor;
	size_t string_length, return_length = 0, res;
	
	conversion_descriptor = iconv_open("ISO-8859-1", "UTF-8");
	if((int) conversion_descriptor < 0) {		
          #ifdef APSRLIB_DEBUG
   	    ERROR("apsr_utf8_decode: conversion descriptor failed!");
          #endif
	}
	
	string_length = strlen(original_string);
	return_length = string_length;
	res = iconv(conversion_descriptor, (char **) &original_string, 
			&string_length, &return_string, &return_length);
	if(res == (size_t) -1) {
	  #ifdef APSRLIB_DEBUG
	    ERROR("apsr_utf8_decode: iconv error occured!");
	  #endif
	}

	if(return_length != 0) {
		*return_string = '?';
		return(RET_FAILURE);
        }

	if(iconv_close(conversion_descriptor) < 0) {
          #ifdef APSRLIB_DEBUG
            ERROR("apsr_utf8_decode: iconv_close failed!");
          #endif
        }	    
	
	return(2-return_length);
}
#else
/* Alternative function for UTF8 decoding */
unsigned char *apsr_utf8_decode(unsigned char *buf) {
	int count=0,count2;

	for(count=0;count<strlen((const char *)buf);count++) {
  	  if((buf[count] & 0x80) == 0) {}
	  else if ((buf[count] & 0x20) == 0) {
	    buf[count]=((( buf[count] & 0x1f ) << 6) | (buf[count+1] & 0x3f));
	    for(count2=count+1;count2<strlen((const char *)buf);count2++)
		    buf[count2]=buf[count2+1];
          }
          else {
            buf[count]=(( buf[count] & 0xf ) << 12) | ( buf[++count] & 0x3f ) << 6 | (buf[count+1] & 0x3f);
	     for(count2=count+1;count2<strlen((const char *)buf);count2++)
		     buf[count2]=buf[count2+1];
           }
	}

	return(buf);
}
#endif


/* Convert int to string */
char * apsr_itoa(int n) {
	int i, sign;
	int c, j;

	char *s;

	s = malloc(36);		/* Should be enough. */

	if ((sign = n) < 0)
		n=-n;

	i = 0;

	do {
		s[i++] = n % 10 + '0';
	} while ((n /= 10) > 0);

	if (sign < 0)
		s[i++] = '-';
	
	s[i] = '\0';

	for (i = 0, j = strlen(s)-1; i < j; i++, j--) {
		c=s[i];
		s[i]=s[j];
		s[j]=c;
	}

	return(s);
}


